"use strict";

var xhr = process.browser ?
/* require('xhr') */
require('request') : require('request');

var https = require('http');

var axios = require('axios');

var inherits = require('util').inherits;

var createPayload = require('../util/create-payload.js');

var Subprovider = require('./subprovider.js');

var _require = require('eth-rpc-errors'),
    ethErrors = _require.ethErrors,
    serializeError = _require.serializeError;

var _require2 = require('socks-proxy-agent'),
    SocksProxyAgent = _require2.SocksProxyAgent;

module.exports = TorRpcSource;
inherits(TorRpcSource, Subprovider);

function TorRpcSource(opts) {
  var self = this;
  self.rpcUrl = opts.rpcUrl;
  self.torProxyUrl = opts.torProxyUrl;
}

TorRpcSource.prototype.handleRequest = function (payload, next, end) {
  var self = this;
  var targetUrl = self.rpcUrl;
  var torProxyUrl = self.torProxyUrl; // overwrite id to conflict with other concurrent users

  var sanitizedPayload = sanitizePayload(payload);
  var newPayload = createPayload(sanitizedPayload);
  var agent = new SocksProxyAgent(torProxyUrl);
  var inst = axios.create({
    httpAgent: agent,
    httpsagent: agent
  });
  inst({
    url: targetUrl,
    method: 'POST',
    headers: {
      'Accept': 'application/json',
      'Content-Type': 'application/json'
    },
    timeout: 60000,
    data: JSON.stringify(newPayload)
  }).then(function (res) {
    // console.log("res.status: " + res.status)
    // check for error code
    switch (res.status) {
      case 405:
        return end(ethErrors.rpc.methodNotFound());

      case 504:
        // Gateway timeout
        return function () {
          var msg = "Gateway timeout. The request took too long to process. ";
          msg += "This can happen when querying logs over too wide a block range.";
          var err = new Error(msg);
          return end(serializeError(err));
        }();

      case 429:
        // Too many requests (rate limiting)
        return function () {
          var err = new Error("Too Many Requests");
          return end(serializeError(err));
        }();

      default:
        // console.log(res)
        if (res.status != 200) {
          var msg = 'Unknown Error: ' + res.data;
          var err = new Error(msg);
          return end(serializeError(err));
        }

    } // parse response


    var data = res.data;
    if (data.error) return end(data.error);
    end(null, data.result);
  })["catch"](function (err) {
    console.error(err.stack);
    return end(serializeError(err));
  });
}; // drops any non-standard params


function sanitizePayload(payload) {
  return {
    id: payload.id,
    jsonrpc: payload.jsonrpc,
    method: payload.method,
    params: payload.params
  };
}